%
% carry out evaluation of segmentation accuracy
%
% copyright(c) 2011  Yihang Bo <yihang.bo@gmail.com>
%

function accuracies = BOevalseg()
resdir = '/.../';  %The path of your segmentation results.
gtdir  = '/.../';  %The path of the groundtruth segmentation.
classes = {'background', 'foreground'};  %Only background and foreground.
% classes = {'background', 'top', 'mid', 'bot'};  %For three parts
% classes = {'background', 'hair', 'face', 'UC', 'arms', 'LC', 'legs'};
% %For each single part.
num = numel(classes);

Dgt  = dir(fullfile(gtdir , '*.png'));
Dres = dir(fullfile(resdir, '*.png'));

assert(numel(Dgt) == numel(Dres));

confcounts = zeros(num);
for i = 1:numel(Dgt)
    gtim  = imread(fullfile(gtdir , Dgt (i).name));
    resim = imread(fullfile(resdir, Dres(i).name));

    maxlabel = max(resim(:));
    if maxlabel > num-1
        error('Result image %d has out of range value %d (should be <= %d)', i, maxlabel, num-1);
    end

    szgtim = size(gtim);
    szresim = size(resim);
    if any(szgtim ~= szresim)
        error('Result image %d is the wrong size, was %d x %d, should be %d x %d', ...
            i, szresim(1), szresim(2), szgtim(1), szgtim(2));
    end

    sumim = gtim + resim*num;
    hs = histc(sumim(:), 0:num*num-1);
    confcounts(:) = confcounts(:) + hs(:);
end

accuracies = zeros(num,1);
fprintf('Accuracy for each class (intersection/union measure)\n');
for j = 1:num
    gtj = sum(confcounts(j,:));  % gt class = j, res class = 0:num-1
    resj = sum(confcounts(:,j)); % gt class = 0:num-1, res class = j
    gtjresj = confcounts(j,j);

    accuracies(j) = 100 * gtjresj / (gtj+resj-gtjresj);

    clname = classes{j};
    fprintf('  %14s: %6.3f%%\n', clname, accuracies(j));
end

avacc = mean(accuracies);
fprintf('-------------------------\n');
fprintf('Average accuracy: %6.3f%%\n',avacc);
